package Logging;

use strict;
use POSIX;
use Carp;
use XmlLogger;

use vars qw|$verbosity $xmlLogger|;
$verbosity = 5;

my %handlerProc;

sub setXmlLogging {
  $xmlLogger = XmlLogger->new();
}

sub setVerbosity {
  $verbosity = shift;
}

sub getVerbosity {
  return $verbosity;
}

sub setOnError {
 $handlerProc{onerror} = shift;
}

sub setOnWarning {
 $handlerProc{onwarning} = shift;
}

my $outputHandle;

sub _uuidgen {
  # http://www.ietf.org/rfc/rfc4122.txt
  my @chars = split(//, 'xxxxxxxx-xxxx-4xxx-8xxx-xxxxxxxxxxxx');
  my @digs  = split(//, '0123456789abcdef');
  my $uuid = '';
  foreach my $char (@chars) {
    if($char eq 'x'){
      $uuid.= $digs[rand(scalar(@digs))];
    }
    else {
      $uuid.= $char;
    }
  }
  return $uuid;
}

sub setOutput {
  my ($fh) = @_;
  $outputHandle = $fh;
}

sub getOutput{
  return $outputHandle if defined $outputHandle;
  return \*STDERR;
}

sub date {
  return POSIX::strftime("%T", gmtime(time()));
}

sub _getMessage {
  my ($marker, $uuid, $message, $code) = @_;
  my $msg = date(). " $marker ";
  $msg .= "$uuid " if defined($uuid);
  $msg .= $message;
  if ( defined $code ) {
    if (( $code eq 'assert') || ($code eq 'fatal')) {
      #local $Carp::CarpLevel = +1;
      $msg .= ": \n" . Carp::longmess();
    }
  }
  $msg .= "\n";
  return $msg;
}

sub error {
  my $uuid = _uuidgen();
  if ( $verbosity > 0 ) {
    my $fh = getOutput();
    print $fh (_getMessage('ERROR',$uuid, @_));
    #print $fh ( date(), " ERROR ", "$uuid ", $_[0], "\n" ); 
  }
  __addXmlMessage('error',$uuid, @_);
  $handlerProc{onerror}->( @_ ) if exists $handlerProc{onerror} and ref $handlerProc{onerror} eq 'CODE';
}

sub warning {
  my $uuid = _uuidgen();
  if ( $verbosity > 1) {
    my $fh = getOutput();
    print $fh (_getMessage('WARN',$uuid, @_));
    #print $fh ( date(), " WARN  ", "$uuid ", $_[0], "\n" );
  }
  __addXmlMessage('warning',$uuid, @_);
  $handlerProc{onwarning}->( @_ ) if exists $handlerProc{onwarning} and ref $handlerProc{onwarning} eq 'CODE';
}

sub info {
  if ( $verbosity > 2) {
    my $fh = getOutput();
    print $fh (_getMessage('INFO',undef, @_));
    #print $fh ( date(), " INFO  ", shift, " ", @_, "\n" );
  }
}

sub debug {
  if ( $verbosity > 3) {
    my $fh = getOutput();
    print $fh (_getMessage('DEBUG',undef, @_));
    #print $fh ( date(), " DEBUG ", shift, " ", @_, "\n" );
  }
}

sub trace {
  if ( $verbosity > 4) {
    my $fh = getOutput();
    print $fh (_getMessage('TRACE',undef, @_));
    #print $fh ( date(), " TRACE ", @_, "\n" );
  }
}

sub beginObject {
  my ( $type, $name ) = @_;
  if( ref($xmlLogger) =~ /XmlLogger/ ) {
    $xmlLogger->beginObject( $type, $name);
  }
}

sub endObject {
  if( ref($xmlLogger) =~ /XmlLogger/ ) {
    $xmlLogger->endObject();
  }
}

sub __addXmlMessage {
  if( ref($xmlLogger) =~ /XmlLogger/ ) {
    $xmlLogger->addMessage( @_);
  }
}

sub serializeXmlLog {
  my ($filename) = @_;
  if( ref($xmlLogger) =~ /XmlLogger/ ) {
    $xmlLogger->serializeToFile( $filename, 1);
  }
}


1;